import { LarkAction, LarkEvent } from "../types"

class LarkBody {
  protected body: LarkEvent.Data | LarkAction.Data
  /**
   * 是否为事件消息
   */
  public isEvent: boolean = false
  /**
   * 是否为Action消息
   */
  public isAction: boolean = false
  /**
   * 事件类型
   */
  public eventType?: LarkEvent.Data["header"]["event_type"]
  /**
   * 是否为消息事件
   */
  public isMessageEvent?: boolean
  /**
   * 消息类型
   */
  public msgType?: LarkEvent.Message["message_type"]
  /**
   * 用户Id
   */
  public userId?: LarkEvent.UserIdInfo["user_id"]
  /**
   * 用户OpenId
   */
  public openId?: LarkEvent.UserIdInfo["open_id"]
  /**
   * 文本内容
   */
  public msgText: string = ""
  /**
   * 原始文本内容
   */
  public rawMsgText: string = ""
  /**
   * 聊天类型
   */
  public chatType?: LarkEvent.Message["chat_type"]
  /**
   * 是否为P2P聊天
   */
  public isP2P?: boolean
  /**
   * 是否在群聊中
   */
  public isInGroup?: boolean
  /**
   * 艾特信息
   */
  public mentions?: LarkEvent.Mention[]
  /**
   * 艾特的机器人
   */
  public mentionedRobot?: LarkEvent.Mention
  /**
   * Action类型
   */
  public actionType?: LarkAction.Data["action"]["tag"]
  /**
   * Action参数
   */
  public actionValue?: LarkAction.Data["action"]["value"]
  /**
   * Action选项
   */
  public actionOption?: LarkAction.Data["action"]["option"]
  /**
   * 对话流Id
   */
  public chatId:
    | LarkEvent.Message["chat_id"]
    | LarkAction.Data["open_chat_id"] = ""
  /**
   * 消息Id
   */
  public messageId:
    | LarkEvent.Message["message_id"]
    | LarkAction.Data["open_message_id"] = ""

  /**
   * 构造函数
   * @param body 事件消息体或Action消息体
   */
  constructor(body: LarkEvent.Data | LarkAction.Data) {
    this.body = body
    this.isEvent = this.getIsEvent(body)
    this.isAction = this.getIsAction(body)

    if (this.isEvent) {
      const eventBody = body as LarkEvent.Data
      this.eventType = this.getEventType(eventBody)
      this.isMessageEvent = this.getIsMessageEvent(eventBody)
      this.msgType = this.getMsgType(eventBody)
      this.userId = this.getUserId(eventBody)
      this.openId = this.getOpenId(eventBody)
      this.msgText = this.getMsgText(eventBody)
      this.rawMsgText = this.getRawMsgText(eventBody)
      this.chatType = this.getChatType(eventBody)
      this.isP2P = this.chatType === "p2p"
      this.isInGroup = this.chatType === "group"
      this.mentions = this.getMentions(eventBody)
      this.mentionedRobot = this.getMentionedRobot(eventBody)
    }

    if (this.isAction) {
      const actionBody = body as LarkAction.Data
      this.actionType = this.getActionType(actionBody)
      this.actionValue = this.getActionValue(actionBody)
      this.actionOption = this.getActionOption(actionBody)
    }

    this.chatId = this.getChatId(body)
    this.messageId = this.getMessageId(body)
  }

  /**
   * 获取事件类型
   * @param body 事件消息体
   * @returns 事件类型
   */
  private getEventType(body: LarkEvent.Data) {
    return body?.header?.event_type
  }

  /**
   * 是否为消息事件
   * @param body 消息事件体
   * @returns
   */
  private getIsMessageEvent(body: LarkEvent.Data) {
    return body?.header?.event_type === "im.message.receive_v1"
  }

  /**
   * 是否为事件消息
   * @param body 事件消息体
   * @returns 是否为事件消息
   */
  private getIsEvent(body: any): body is LarkEvent.Data {
    return !!body?.event
  }

  /**
   * 是否为Action消息
   * @param body Action消息体
   * @returns 是否为Action消息
   */
  private getIsAction(body: any): body is LarkAction.Data {
    return !!body?.action
  }

  /**
   * 获取事件文本类型
   * @param body 事件消息体
   * @returns 事件文本类型
   */
  private getMsgType(body: LarkEvent.Data) {
    return body?.event?.message?.message_type
  }

  /**
   * 获取用户Id
   * @param body 事件消息体
   * @returns 用户Id
   */
  private getUserId(body: LarkEvent.Data) {
    return body?.event?.sender?.sender_id?.user_id
  }

  /**
   * 获取用户OpenId
   * @param body 事件消息体
   * @returns 用户OpenId
   */
  private getOpenId(body: LarkEvent.Data) {
    return body?.event?.sender?.sender_id?.open_id
  }

  /**
   * 获取文本内容并剔除艾特信息
   * @param body 事件消息体
   * @returns 文本内容
   */
  private getMsgText(body: LarkEvent.Data) {
    try {
      const { text }: { text: string } = JSON.parse(
        body?.event?.message?.content
      )
      // 去掉@_user_1相关的内容，例如 '@_user_1 测试' -> '测试'
      const textWithoutAt = text.replace(/@_user_\d+/g, "")
      return textWithoutAt.trim()
    } catch {
      return ""
    }
  }

  /**
   * 获取原始文本内容
   * @param body 事件消息体
   * @returns 原始文本内容
   */
  private getRawMsgText(body: LarkEvent.Data) {
    try {
      const { text }: { text: string } = JSON.parse(
        body?.event?.message?.content
      )
      return text
    } catch {
      return ""
    }
  }

  /**
   * 获取聊天类型
   * @param body 事件消息体
   * @returns 聊天类型
   */
  private getChatType(body: LarkEvent.Data) {
    return body?.event?.message?.chat_type
  }

  /**
   * 获取艾特信息
   * @param body 事件消息体
   * @returns 艾特信息
   */
  private getMentions(body: LarkEvent.Data) {
    return body?.event?.message?.mentions
  }

  /**
   * 获取艾特的机器人
   * @param body 事件消息体
   * @returns 艾特的机器人
   */
  private getMentionedRobot(body: LarkEvent.Data) {
    const mentions = this.getMentions(body)
    // 返回第一个没有user_id的mention
    return mentions?.find((mention) => !mention.id?.user_id)
  }

  /**
   * 获取Action类型
   * @param body Action消息体
   * @returns Action类型
   */
  private getActionType(body: LarkAction.Data) {
    return body?.action?.tag
  }

  /**
   * 获取Action参数
   * @param body Action消息体
   * @returns Action参数
   */
  private getActionValue(body: LarkAction.Data) {
    return body?.action?.value
  }

  /**
   * 获取Action选项
   * @param body Action消息体
   * @returns Action选项
   */
  private getActionOption(body: LarkAction.Data) {
    return body?.action?.option
  }

  /**
   * 获取对话流Id
   * @param body 事件消息体或Action消息体
   * @returns 对话流Id
   */
  private getChatId(body: LarkEvent.Data | LarkAction.Data) {
    if (this.getIsEvent(body)) return body?.event?.message?.chat_id
    if (this.getIsAction(body)) return body?.open_chat_id
    return ""
  }

  /**
   * 获取消息Id
   * @param body 事件消息体或Action消息体
   * @returns 消息Id
   */
  private getMessageId(body: LarkEvent.Data | LarkAction.Data) {
    if (this.getIsEvent(body)) return body?.event?.message?.message_id
    if (this.getIsAction(body)) return body?.open_message_id
    return ""
  }

  /**
   * 是否艾特了机器人
   * @param name 机器人名字
   * @returns 是否艾特了机器人
   */
  public isAtBot(name: string) {
    if (!this.mentions) return false
    if (!this.isInGroup) return false
    return this.mentions.some((m) => m.name === name)
  }
}

export default LarkBody
